<!doctype html>
<html>
  <head>
    <meta charset="utf-8">
    <title>tree</title>
    <meta name="description" content="">
    <meta name="viewport" content="width=device-width">
    <script src="https://code.jquery.com/jquery-latest.min.js" type="text/javascript"></script>
    <script src="https://d3js.org/d3.v3.min.js" type="text/javascript"></script>
    <style>
        #tree-container {
	position: absolute;
	left: 0px;
	width: 100%;
}
.svgContainer {
	display: block;
    margin: auto;
}
.node {
	cursor: pointer;
}
.node-rect-closed {
	stroke-width: 2px;
	stroke: rgb(0,0,0);
}
.link {
	fill: none;
	stroke: lightsteelblue;
	stroke-width: 2px;
}
.linkselected {
	fill: none;
	stroke: tomato;
	stroke-width: 2px;
}
.arrow {
	fill: lightsteelblue;
	stroke-width: 1px;
}
.arrowselected {
	fill: tomato;
	stroke-width: 2px;
}
.link text {
	font: 7px sans-serif;
	fill: #CC0000;
}
.wordwrap {
	white-space: pre-wrap; /* CSS3 */
	white-space: -moz-pre-wrap; /* Firefox */
	white-space: -pre-wrap; /* Opera <7 */
	white-space: -o-pre-wrap; /* Opera 7 */
	word-wrap: break-word; /* IE */
}
.node-text {
	font: 7px sans-serif;
	color: white;
}
.tooltip-text-container {
    height: 100%;
	width: 100%;
}
.tooltip-text {
	visibility: hidden;
	font: 7px sans-serif;
	color: white;
	display: block;
 	padding: 5px;
}
.tooltip-box {
	background: rgba(0, 0, 0, 0.7);
	visibility: hidden;
	position: absolute;
	border-style: solid;
    border-width: 1px;
    border-color: black;
    border-top-right-radius: 0.5em;
}
p {
	display: inline;
}
.textcolored {
	color: orange;
}
a.exchangeName {
	color: orange;
}
#tdata{
    width:300px;
}
#tables{
    position:absolute;
    bottom:0% !important;
}
    </style>
  </head>
  <body>
    <div class="container" style='width:100%'>
        <ct-visualization id="tree-container"></ct-visualization>
        <script>
            d3.json("./static/tree2.json", function(error, json) {
                treeBoxes('', json.tree);
            });
        </script>
    </div>
  <div id'tables'>
    <div id='pdata'>
        <p id='file_name'></p><br>
        <p id='routes' ></p><br>
        <p id='functions'></p>
    </div>
     <div id='tdata'>
     <tr>
        <th>Tag</th>
        <th>Counts</th>
      </tr>
    <table id='table' >
     </table>
    </div>
</div>
</body>
<script>
// var filedata = JSON.parse('{{ data| safe}}')
// console.log(filedata)
// var k = Object.keys(filedata.templates)
// var v = Object.values(filedata.templates)
// console.log(k)
 $('#tdata').hide()
function treeBoxes(urlService, jsonData)
{
	var urlService_ = '';
	var blue = '#337ab7',
		green = '#5cb85c',
		yellow = '#f0ad4e',
		blueText = '#4ab1eb',
		purple = '#9467bd';
        red="#D72822"
	var margin = {
					top : 0,
					right : 0,
					bottom : 100,
					left : 0
				 },
		// Height and width are redefined later in function of the size of the tree
		// (after that the data are loaded)
		width = 800 - margin.right - margin.left,
		height = 400 - margin.top - margin.bottom;
	var rectNode = { width : 120, height : 45, textMargin : 5 },
		tooltip = { width : 150, height : 40, textMargin : 5 };
	var i = 0,
		duration = 750,
		root;
	var mousedown; // Use to save temporarily 'mousedown.zoom' value
	var mouseWheel,
		mouseWheelName,
		isKeydownZoom = false;
	var tree;
	var baseSvg,
		svgGroup,
		nodeGroup, // If nodes are not grouped together, after a click the svg node will be set after his corresponding tooltip and will hide it
		nodeGroupTooltip,
		linkGroup,
		linkGroupToolTip,
		defs;
	init(urlService, jsonData);
	function init(urlService, jsonData)
	{
		urlService_ = urlService;
		if (urlService && urlService.length > 0)
		{
			if (urlService.charAt(urlService.length - 1) != '/')
				urlService_ += '/';
		}
		if (jsonData)
			drawTree(jsonData);
		else
		{
			console.error(jsonData);
			alert('Invalides data.');
		}
	}
	function drawTree(jsonData)
	{
		tree = d3.layout.tree().size([ height, width ]);
		root = jsonData;
		root.fixed = true;
		// Dynamically set the height of the main svg container
		// breadthFirstTraversal returns the max number of node on a same level
		// and colors the nodes
		var maxDepth = 0;
		var maxTreeWidth = breadthFirstTraversal(tree.nodes(root), function(currentLevel) {
			maxDepth++;
			currentLevel.forEach(function(node) {
				if (node.type == 'type1')
					node.color = blue;
				if (node.type == 'type2')
					node.color = green;
				if (node.type == 'type3')
					node.color = purple;
				if (node.type == 'type4')
					node.color = yellow;
                if (node.type == 'type5')
					node.color = red;
				});
			});
		height = maxTreeWidth * (rectNode.height + 20) + tooltip.height + 20 - margin.right - margin.left;
		width = maxDepth * (rectNode.width * 1.5) + tooltip.width / 2 - margin.top - margin.bottom;
		tree = d3.layout.tree().size([ height, width ]);
		root.x0 = height / 2;
		root.y0 = 0;
		baseSvg = d3.select('#tree-container').append('svg')
	    .attr('width', width + margin.right + margin.left)
		.attr('height', height + margin.top + margin.bottom)
		.attr('class', 'svgContainer')
		.call(d3.behavior.zoom()
		      //.scaleExtent([0.5, 1.5]) // Limit the zoom scale
		      .on('zoom', zoomAndDrag));
		// Mouse wheel is desactivated, else after a first drag of the tree, wheel event drags the tree (instead of scrolling the window)
		getMouseWheelEvent();
		d3.select('#tree-container').select('svg').on(mouseWheelName, null);
		d3.select('#tree-container').select('svg').on('dblclick.zoom', null);
		svgGroup = baseSvg.append('g')
		.attr('class','drawarea')
		.append('g')
		.attr('transform', 'translate(' + margin.left + ',' + margin.top + ')');
		// SVG elements under nodeGroupTooltip could be associated with nodeGroup,
		// same for linkGroupToolTip and linkGroup,
		// but this separation allows to manage the order on which elements are drew
		// and so tooltips are always on top.
		nodeGroup = svgGroup.append('g')
					.attr('id', 'nodes');
		linkGroup = svgGroup.append('g')
					.attr('id', 'links');
		linkGroupToolTip = svgGroup.append('g')
			   				.attr('id', 'linksTooltips');
		nodeGroupTooltip = svgGroup.append('g')
			   				.attr('id', 'nodesTooltips');
		defs = baseSvg.append('defs');
		initArrowDef();
		initDropShadow();
		update(root);
	}
	function update(source)
	{
		// Compute the new tree layout
		var nodes = tree.nodes(root).reverse(),
			links = tree.links(nodes);
		// Check if two nodes are in collision on the ordinates axe and move them
		breadthFirstTraversal(tree.nodes(root), collision);
		// Normalize for fixed-depth
		nodes.forEach(function(d) {
			d.y = d.depth * (rectNode.width * 1.5);
		});
	// 1) ******************* Update the nodes *******************
		var node = nodeGroup.selectAll('g.node').data(nodes, function(d) {
			return d.id || (d.id = ++i);
		});
		var nodesTooltip = nodeGroupTooltip.selectAll('g').data(nodes, function(d) {
			return d.id || (d.id = ++i);
		});
		// Enter any new nodes at the parent's previous position
		// We use "insert" rather than "append", so when a new child node is added (after a click)
		// it is added at the top of the group, so it is drawed first
		// else the nodes tooltips are drawed before their children nodes and they
		// hide them
		var nodeEnter = node.enter().insert('g', 'g.node')
		.attr('class', 'node')
		.attr('transform', function(d) {
			  return 'translate(' + source.y0 + ',' + source.x0 + ')'; })
		.on('click', function(d) {
						click(d);
					if(d.typed == 'File'){
					    console.log(d.nodeName)
					    if(d.nodeName.includes('.html')){
                          $('#table').text('')
                          $('#file_name').text('')
                          $('#routes').text('')
                          $('#functions').text('')
                          $('#tdata').show()
                            var keys = Object.keys(filedata['templates'][d.nodeName])
                            var values = Object.values(filedata['templates'][d.nodeName])
                              for (x=0;x<keys.length;x++){
                                  keys[x] = keys[x].replace('<', '')
                                //   str = keys[x][0]
                                //   str.replace('<' , '')
                                    $('#table').append('<tr><td>'+keys[x]+'</td><td>'+values[x]+'</td></tr>')
                                }
					    }
					     if(d.nodeName.includes('.py')){
					       let f = d.nodeName
					       $('#table').text('')
                            $('#tdata').hide()
					       $('#file_name').text('File name: '+ f)
					       $('#functions').text('Function: ' + filedata['python'][f].function)
					       $('#routes').text('Routes: ' + filedata['python'][f].route)
					        console.log(filedata['python'][f])
					    }
					}
			});
		var nodeEnterTooltip = nodesTooltip.enter().append('g')
			.attr('transform', function(d) {
				  return 'translate(' + source.y0 + ',' + source.x0 + ')'; });
		nodeEnter.append('g').append('rect')
		.attr('rx', 7)
		.attr('ry', 7)
		.attr('width', rectNode.width)
		.attr('height', rectNode.height)
		.attr('class', 'node-rect')
		.attr('fill', function (d) { return d.color; })
		.attr('filter', 'url(#drop-shadow)');
		nodeEnter.append('foreignObject')
		.attr('x', rectNode.textMargin)
		.attr('y', rectNode.textMargin)
		.attr('width', function() {
					return (rectNode.width - rectNode.textMargin * 2) < 0 ? 0
							: (rectNode.width - rectNode.textMargin * 2)
				})
		.attr('height', function() {
					return (rectNode.height - rectNode.textMargin * 2) < 0 ? 0
							: (rectNode.height - rectNode.textMargin * 2)
				})
		.append('xhtml').html(function(d) {
					return '<div style="width: '
							+ (rectNode.width - rectNode.textMargin * 2) + 'px; height: '
							+ (rectNode.height - rectNode.textMargin * 2) + 'px;" class="node-text wordwrap">'
							+ '<p style="font-size:14px;">' + d.nodeName + '</p><br><br>'
                            +'<p style="font-size:11px;">' + d.typed + '</p><br><br>'
							+ '</div>';
				});
		nodeEnterTooltip.append("rect")
		.attr('id', function(d) { return 'nodeInfoID' + d.id; })
    	.attr('x', rectNode.width / 2)
		.attr('y', rectNode.height / 2)
		.attr('width', tooltip.width)
		.attr('height', tooltip.height)
    	.attr('class', 'tooltip-box')
    	.style('fill-opacity', 0.8)
		.on('mouseover', function(d) {
			$('#nodeInfoID' + d.id).css('visibility', 'visible');
			$('#nodeInfoTextID' + d.id).css('visibility', 'visible');
			removeMouseEvents();
		})
		.on('mouseout', function(d) {
			$('#nodeInfoID' + d.id).css('visibility', 'hidden');
			$('#nodeInfoTextID' + d.id).css('visibility', 'hidden');
			reactivateMouseEvents();
		});
		nodeEnterTooltip.append("text")
		.attr('id', function(d) { return 'nodeInfoTextID' + d.id; })
    	.attr('x', rectNode.width / 2 + tooltip.textMargin)
		.attr('y', rectNode.height / 2 + tooltip.textMargin * 2)
		.attr('width', tooltip.width)
		.attr('height', tooltip.height)
		.attr('class', 'tooltip-text')
		.style('fill', 'white')
		.append("tspan")
	    .text(function(d) {return 'Name: ' + d.name;})
	    .append("tspan")
	    .attr('x', rectNode.width / 2 + tooltip.textMargin)
	    .attr('dy', '1.5em')
	    .text(function(d) {return 'Info: ' + d.label;});
		// Transition nodes to their new position.
		var nodeUpdate = node.transition().duration(duration)
		.attr('transform', function(d) { return 'translate(' + d.y + ',' + d.x + ')'; });
		nodesTooltip.transition().duration(duration)
		.attr('transform', function(d) { return 'translate(' + d.y + ',' + d.x + ')'; });
		nodeUpdate.select('rect')
		.attr('class', function(d) { return d._children ? 'node-rect-closed' : 'node-rect'; });
		nodeUpdate.select('text').style('fill-opacity', 1);
		// Transition exiting nodes to the parent's new position
		var nodeExit = node.exit().transition().duration(duration)
			.attr('transform', function(d) { return 'translate(' + source.y + ',' + source.x + ')'; })
			.remove();
		nodesTooltip.exit().transition().duration(duration)
			.attr('transform', function(d) { return 'translate(' + source.y + ',' + source.x + ')'; })
		.remove();
		nodeExit.select('text').style('fill-opacity', 1e-6);
	// 2) ******************* Update the links *******************
		var link = linkGroup.selectAll('path').data(links, function(d) {
			return d.target.id;
		});
		var linkTooltip = linkGroupToolTip.selectAll('g').data(links, function(d) {
			return d.target.id;
		});
		function linkMarkerStart(direction, isSelected) {
			if (direction == 'SYNC')
			{
				return isSelected ? 'url(#start-arrow-selected)' : 'url(#start-arrow)';
			}
			return '';
		}
		function linkType(link) {
			if (link.direction == 'SYNC')
				return "Synchronous [\u2194]";
			else
			{
				if (link.direction == 'ASYN')
					return "Asynchronous [\u2192]";
			}
			return '???';
		}
		d3.selection.prototype.moveToFront = function() {
			  return this.each(function(){
				    this.parentNode.appendChild(this);
				  });
			};
		// Enter any new links at the parent's previous position.
			// Enter any new links at the parent's previous position.
			var linkenter = link.enter().insert('path', 'g')
			.attr('class', 'link')
			.attr('id', function(d) { return 'linkID' + d.target.id; })
			.attr('d', function(d) { return diagonal(d); })
			.attr('marker-end', 'url(#end-arrow)')
			.attr('marker-start', function(d) { return linkMarkerStart(d.target.link.direction, false); })
			.on('mouseover', function(d) {
				d3.select(this).moveToFront();
				d3.select(this).attr('marker-end', 'url(#end-arrow-selected)');
				d3.select(this).attr('marker-start', linkMarkerStart(d.target.link.direction, true));
				d3.select(this).attr('class', 'linkselected');
				$('#tooltipLinkID' + d.target.id).attr('x', (d.target.y + rectNode.width - d.source.y) / 2 + d.source.y);
				$('#tooltipLinkID' + d.target.id).attr('y', (d.target.x - d.source.x) / 2 + d.source.x);
				$('#tooltipLinkID' + d.target.id).css('visibility', 'visible');
				$('#tooltipLinkTextID' + d.target.id).css('visibility', 'visible');
			})
			.on('mouseout', function(d) {
				d3.select(this).attr('marker-end', 'url(#end-arrow)');
				d3.select(this).attr('marker-start', linkMarkerStart(d.target.link.direction, false));
				d3.select(this).attr('class', 'link');
				$('#tooltipLinkID' + d.target.id).css('visibility', 'hidden');
				$('#tooltipLinkTextID' + d.target.id).css('visibility', 'hidden');
			});
			linkTooltip.enter().append('rect')
			.attr('id', function(d) { return 'tooltipLinkID' + d.target.id; })
			.attr('class', 'tooltip-box')
			.style('fill-opacity', 0.8)
			.attr('x', function(d) { return (d.target.y + rectNode.width - d.source.y) / 2 + d.source.y; })
			.attr('y', function(d) { return (d.target.x - d.source.x) / 2 + d.source.x; })
			.attr('width', tooltip.width)
			.attr('height', tooltip.height)
			.on('mouseover', function(d) {
				$('#tooltipLinkID' + d.target.id).css('visibility', 'visible');
				$('#tooltipLinkTextID' + d.target.id).css('visibility', 'visible');
				// After selected a link, the cursor can be hover the tooltip, that's why we still need to highlight the link and the arrow
				$('#linkID' + d.target.id).attr('class', 'linkselected');
				$('#linkID' + d.target.id).attr('marker-end', 'url(#end-arrow-selected)');
				$('#linkID' + d.target.id).attr('marker-start', linkMarkerStart(d.target.link.direction, true));
				removeMouseEvents();
			})
			.on('mouseout', function(d) {
				$('#tooltipLinkID' + d.target.id).css('visibility', 'hidden');
				$('#tooltipLinkTextID' + d.target.id).css('visibility', 'hidden');
				$('#linkID' + d.target.id).attr('class', 'link');
				$('#linkID' + d.target.id).attr('marker-end', 'url(#end-arrow)');
				$('#linkID' + d.target.id).attr('marker-start', linkMarkerStart(d.target.link.direction, false));
				reactivateMouseEvents();
			});
			linkTooltip.enter().append('text')
			.attr('id', function(d) { return 'tooltipLinkTextID' + d.target.id; })
			.attr('class', 'tooltip-text')
			.attr('x', function(d) { return (d.target.y + rectNode.width - d.source.y) / 2 + d.source.y + tooltip.textMargin; })
			.attr('y', function(d) { return (d.target.x - d.source.x) / 2 + d.source.x + tooltip.textMargin * 2; })
			.attr('width', tooltip.width)
			.attr('height', tooltip.height)
			.style('fill', 'white')
			.append("tspan")
	   		.text(function(d) { return linkType(d.target.link); })
	   		.append("tspan")
	    	.attr('x', function(d) { return (d.target.y + rectNode.width - d.source.y) / 2 + d.source.y + tooltip.textMargin; })
	   		.attr('dy', '1.5em')
	    	.text(function(d) {return d.target.link.name;});
		// Transition links to their new position.
		var linkUpdate = link.transition().duration(duration)
						 	 .attr('d', function(d) { return diagonal(d); });
		linkTooltip.transition().duration(duration)
				   .attr('d', function(d) { return diagonal(d); });
		// Transition exiting nodes to the parent's new position.
		link.exit().transition()
		.remove();
		linkTooltip.exit().transition()
			.remove();
		// Stash the old positions for transition.
		nodes.forEach(function(d) {
			d.x0 = d.x;
			d.y0 = d.y;
		});
	}
	// Zoom functionnality is desactivated (user can use browser Ctrl + mouse wheel shortcut)
	function zoomAndDrag() {
	    //var scale = d3.event.scale,
	    var scale = 1,
	        translation = d3.event.translate,
	        tbound = -height * scale,
	        bbound = height * scale,
	        lbound = (-width + margin.right) * scale,
	        rbound = (width - margin.left) * scale;
	    // limit translation to thresholds
	    translation = [
	        Math.max(Math.min(translation[0], rbound), lbound),
	        Math.max(Math.min(translation[1], bbound), tbound)
	    ];
	    d3.select('.drawarea')
	        .attr('transform', 'translate(' + translation + ')' +
	              ' scale(' + scale + ')');
	}
	// Toggle children on click.
	function click(d) {
		if (d.children) {
			d._children = d.children;
			d.children = null;
		} else {
			d.children = d._children;
			d._children = null;
		}
		update(d);
	}
	// Breadth-first traversal of the tree
	// func function is processed on every node of a same level
	// return the max level
	  function breadthFirstTraversal(tree, func)
	  {
		  var max = 0;
		  if (tree && tree.length > 0)
		  {
			  var currentDepth = tree[0].depth;
			  var fifo = [];
			  var currentLevel = [];
			  fifo.push(tree[0]);
			  while (fifo.length > 0) {
				  var node = fifo.shift();
				  if (node.depth > currentDepth) {
					  func(currentLevel);
					  currentDepth++;
					  max = Math.max(max, currentLevel.length);
					  currentLevel = [];
				  }
				  currentLevel.push(node);
				  if (node.children) {
					  for (var j = 0; j < node.children.length; j++) {
						  fifo.push(node.children[j]);
					  }
				  }
		  	}
			func(currentLevel);
			return Math.max(max, currentLevel.length);
		}
		return 0;
	  }
	// x = ordoninates and y = abscissas
	function collision(siblings) {
	  var minPadding = 5;
	  if (siblings) {
		  for (var i = 0; i < siblings.length - 1; i++)
		  {
			  if (siblings[i + 1].x - (siblings[i].x + rectNode.height) < minPadding)
				  siblings[i + 1].x = siblings[i].x + rectNode.height + minPadding;
		  }
	  }
	}
	function removeMouseEvents() {
		// Drag and zoom behaviors are temporarily disabled, so tooltip text can be selected
		mousedown = d3.select('#tree-container').select('svg').on('mousedown.zoom');
		d3.select('#tree-container').select('svg').on("mousedown.zoom", null);
	}
	function reactivateMouseEvents() {
		// Reactivate the drag and zoom behaviors
		d3.select('#tree-container').select('svg').on('mousedown.zoom', mousedown);
	}
	// Name of the event depends of the browser
	function getMouseWheelEvent() {
		if (d3.select('#tree-container').select('svg').on('wheel.zoom'))
		{
			mouseWheelName = 'wheel.zoom';
			return d3.select('#tree-container').select('svg').on('wheel.zoom');
		}
		if (d3.select('#tree-container').select('svg').on('mousewheel.zoom') != null)
		{
			mouseWheelName = 'mousewheel.zoom';
			return d3.select('#tree-container').select('svg').on('mousewheel.zoom');
		}
		if (d3.select('#tree-container').select('svg').on('DOMMouseScroll.zoom'))
		{
			mouseWheelName = 'DOMMouseScroll.zoom';
			return d3.select('#tree-container').select('svg').on('DOMMouseScroll.zoom');
		}
	}
	function diagonal(d) {
		var p0 = {
			x : d.source.x + rectNode.height / 2,
			y : (d.source.y + rectNode.width)
		}, p3 = {
			x : d.target.x + rectNode.height / 2,
			y : d.target.y  - 12 // -12, so the end arrows are just before the rect node
		}, m = (p0.y + p3.y) / 2, p = [ p0, {
			x : p0.x,
			y : m
		}, {
			x : p3.x,
			y : m
		}, p3 ];
		p = p.map(function(d) {
			return [ d.y, d.x ];
		});
		return 'M' + p[0] + 'C' + p[1] + ' ' + p[2] + ' ' + p[3];
	}
	function initDropShadow() {
		var filter = defs.append("filter")
		    .attr("id", "drop-shadow")
		    .attr("color-interpolation-filters", "sRGB");
		filter.append("feOffset")
		.attr("result", "offOut")
		.attr("in", "SourceGraphic")
	    .attr("dx", 0)
	    .attr("dy", 0);
		filter.append("feGaussianBlur")
		    .attr("stdDeviation", 2);
		filter.append("feOffset")
		    .attr("dx", 2)
		    .attr("dy", 2)
		    .attr("result", "shadow");
		filter.append("feComposite")
	    .attr("in", 'offOut')
	    .attr("in2", 'shadow')
	    .attr("operator", "over");
	}
	function initArrowDef() {
		// Build the arrows definitions
		// End arrow
		defs.append('marker')
		.attr('id', 'end-arrow')
		.attr('viewBox', '0 -5 10 10')
		.attr('refX', 0)
		.attr('refY', 0)
		.attr('markerWidth', 6)
		.attr('markerHeight', 6)
		.attr('orient', 'auto')
		.attr('class', 'arrow')
		.append('path')
		.attr('d', 'M0,-5L10,0L0,5');
		// End arrow selected
		defs.append('marker')
		.attr('id', 'end-arrow-selected')
		.attr('viewBox', '0 -5 10 10')
		.attr('refX', 0)
		.attr('refY', 0)
		.attr('markerWidth', 6)
		.attr('markerHeight', 6)
		.attr('orient', 'auto')
		.attr('class', 'arrowselected')
		.append('path')
		.attr('d', 'M0,-5L10,0L0,5');
		// Start arrow
		defs.append('marker')
		.attr('id', 'start-arrow')
		.attr('viewBox', '0 -5 10 10')
		.attr('refX', 0)
		.attr('refY', 0)
		.attr('markerWidth', 6)
		.attr('markerHeight', 6)
		.attr('orient', 'auto')
		.attr('class', 'arrow')
		.append('path')
		.attr('d', 'M10,-5L0,0L10,5');
		// Start arrow selected
		defs.append('marker')
		.attr('id', 'start-arrow-selected')
		.attr('viewBox', '0 -5 10 10')
		.attr('refX', 0)
		.attr('refY', 0)
		.attr('markerWidth', 6)
		.attr('markerHeight', 6)
		.attr('orient', 'auto')
		.attr('class', 'arrowselected')
		.append('path')
		.attr('d', 'M10,-5L0,0L10,5');
	}
}
					
</script>
</html>


import os
import json
import requests



ROOT = os.path.dirname(os.path.abspath(__file__))
sample_tree_file = os.path.join(ROOT,"sample_tree.py")
def add_numbers(num1,num2):
	return num1+num2

def analyze_data(filename):
	modules = []
	classes = []
	res = {}
	current_class = ''
	data = open(filename)
	for x in data:
		if 'import' in x or 'from' in x:
			modules.append(x)

		if 'class' in x:
			classes.append(x)
			c = x.split(' ')
			if c[0] == 'class':
				current_class = x.strip()
				res[current_class] = {
					'defs': [],
					'classes': []
				}
			# print('new class, ' , x)
			else:
				if current_class != '':
					# print('old class', x)
					res[current_class]['classes'].append(x.strip())

		if 'def' in x:
			if current_class != '':
				res[current_class]['defs'].append(x.strip())

	d = {}

	d['cls'] = res

	d['modules'] = modules
	# obj = json.dumps(res, indent=4)
	data.close()

	return d


def addNode(name, typed, childof, n_deci, children, pos):
	ob = {
		"nodeName": name,
		"name": "NODE NAME {0:.1f}".format(n_deci),
		"type": "type2",
		"typed": "module",

		"link": {
			"name": "Link node {0:.1f} to {1:.1f}".format(childof, n_deci),
			"nodeName": "NODE NAME {0:.1f}".format(n_deci),
			"direction": "SYNC"
		},
		"children": children
	}
	if pos == 'root':
		ob['name'] = name
		ob['link']['nodeName'] = name
		ob['link']['name'] = "Link NODE NAME 1.0"
		ob['type'] = 'type1'
		ob['typed'] = 'module'

	return ob



def get_childs(x):
	n_deci = 1.1
	n = 1.1
	r = addNode(x.split('/')[-1], "type1", n, n_deci, [], 'root')
	n_deci = 1.1
	n = 1.1

	fData = analyze_data(x)
	cl = addNode(x.split('/')[-1], "type2", n, n_deci + 1, [], 'child')
	c = {}
	for cls in fData['cls']:
		c['classes'] = addNode(cls, "type2", n, n_deci + 1, [
			addNode("Classes", "type2", n, n_deci + 2, [], 'child'),
			addNode("functions", "type2", n, n_deci + 2, [], 'child')
		], 'child')
		for x in fData['cls'][cls]['classes']:
			n_deci += 1
			c['classes']['children'][0]['children'].append(addNode(x, "type2", n, n_deci + 3, [], 'child'))
		for x in fData['cls'][cls]['defs']:
			n_deci += 1
			c['classes']['children'][1]['children'].append(addNode(x, "type2", n, n_deci + 3, [], 'child'))
	n_deci += 1
	n = n_deci
	cl['children'] = [c['classes']]
	r['children'].append(cl)
	return cl

def file_traverse(path,output_dir):
    root = get_childs(path)
    with open(sample_tree_file) as fildata:
        data = "".join(fildata.readlines())
        
    data += '<script>treeBoxes("", {0});</script>'.format(root)
    os.chmod(output_dir, 0o777)
    output_dir = os.path.join(output_dir + '/dtree.html')
    
    with open(output_dir, 'w') as f:
        f.write(data)
		
	
    return  {'tree' : root}



def folder_files_traverse(folder,output_dir):
	childrens = []
	files = os.listdir(folder )
	d = folder.split('/')[-1]
	if d == '': d = folder.split('/')[-2]
	for x in os.listdir(folder):
		if x.endswith('.py'):
			childrens.append(get_childs(folder + x))
	root = addNode(d, "type1", 0, 0, childrens, 'root')
	
	with open(sample_tree_file) as fildata:
		data = "".join(fildata.readlines())
		
	data += '<script>treeBoxes("", {0});</script>'.format(root)
	
	os.chmod(output_dir, 0o777)
	output_dir = os.path.join(output_dir + '/dtree.html')
	
	with open('dtree.html', 'w') as f:
		f.write(data)

	
	return {'tree' : root}
	




'''
#====================================EXAMPLE FOR COMPLETE FOLDER===============================
data = folder_files_traverse(folder)
f = open(outputfile, 'w')
print(data)
f.write(json.dumps( data, indent=4))
f.close()

'''
#====================================EXAMPLE FOR SINGLE FILE===============================
#data = file_traverse(filename)
#f = open(outputfile)
#f.write(json.dumps( data, indent=4))
#print(data)
#f.close()

def demo():
	path = os.path.join(ROOT, "sample_python_file.py")
	output_dir = ROOT
	data = file_traverse(path,output_dir)
	print("Demo file create successfully in " + output_dir )




from django.utils import timezone
from django.db import transaction

from wis_queue_manager.models import FileProcessingTask, FileIndexingTask, FileDecompressionTask, FileDiffTask


class QueueManager(object):
    @staticmethod
    def get_next_data_processing_task():
        with transaction.atomic():
            file_processing_task = (
                FileProcessingTask
                .objects
                .select_for_update(skip_locked=True)
                .filter(status=FileProcessingTask.TODO)
                .order_by('processing_priority', 'id')
                .first()
            )
            if file_processing_task:
                file_processing_task.status = FileProcessingTask.IN_PROGRESS
                file_processing_task.start_time = timezone.now()
                file_processing_task.last_updated = file_processing_task.start_time
                file_processing_task.finish_time = None
                file_processing_task.duration = None
                file_processing_task.records_processed = 0
                file_processing_task.records_processed_per_second = None
                file_processing_task.save()
        return file_processing_task

    @staticmethod
    def get_next_data_indexing_task():
        with transaction.atomic():
            file_indexing_task = (
                FileIndexingTask
                    .objects
                    .select_for_update(skip_locked=True)
                    .filter(status = FileIndexingTask.TODO)
                    .first()
            )

            if file_indexing_task:
                file_indexing_task.status = FileIndexingTask.IN_PROGRESS
                file_indexing_task.start_time = timezone.now()
                file_indexing_task.last_updated = file_indexing_task.start_time
                file_indexing_task.finish_time = None
                file_indexing_task.duration = None
                file_indexing_task.records_processed = 0
                file_indexing_task.records_processed_per_second = None
                file_indexing_task.save()
            return file_indexing_task

    @staticmethod
    def get_next_data_decompression_task():
        with transaction.atomic():
            file_decompression_task = (
                FileDecompressionTask
                    .objects
                    .select_for_update(skip_locked=True)
                    .filter(status = FileDecompressionTask.TODO)
                    .order_by('processing_priority', '-file_size')
                    .first()
            )

            if file_decompression_task:
                file_decompression_task.status = FileDecompressionTask.IN_PROGRESS
                file_decompression_task.start_time = timezone.now()
                file_decompression_task.last_updated = file_decompression_task.start_time
                file_decompression_task.finish_time = None
                file_decompression_task.duration = None
                file_decompression_task.records_processed = 0
                file_decompression_task.records_processed_per_second = None
                file_decompression_task.save()
            return file_decompression_task

    @staticmethod
    def get_next_file_diff_task():
        with transaction.atomic():
            file_diff_task = (
                FileDiffTask
                    .objects
                    .select_for_update(skip_locked=True)
                    .filter(status = FileDiffTask.TODO)
                    .first()
            )

            if file_diff_task:
                file_diff_task.status = FileDiffTask.IN_PROGRESS
                file_diff_task.start_time = timezone.now()
                file_diff_task.last_updated = file_diff_task.start_time
                file_diff_task.finish_time = None
                file_diff_task.duration = None
                file_diff_task.records_processed = 0
                file_diff_task.records_processed_per_second = None
                file_diff_task.save()
            return file_diff_task




import os

commands = ["python setup.py sdist", "twine upload -r testpypi dist/* -username='alekya' -password='Kumar2020' --verbose"]
for c in commands:
    stream = os.popen(c)
    output = stream.readlines()
    for x in output:
        print(x)
